from typing import List

import pytest
from pyfakefs.fake_filesystem import FakeFilesystem

from gcs_migration.parsers import BlackPearlMapEntry, BlackPearlAccessIdFileParser


def test_blackpearl_mapfile_parser(
    fs: FakeFilesystem,
    blackpearl_access_id_file: str,
    blackpearl_map_entries: List[BlackPearlMapEntry],
    user_without_mapfile: str,
) -> None:
    map_parser = BlackPearlAccessIdFileParser(blackpearl_access_id_file)

    for entry in blackpearl_map_entries:
        assert entry.username in map_parser
        assert map_parser[entry.username] == entry


def test_blackpearl_mapfile_parser_invalid_user(
    fs: FakeFilesystem,
    blackpearl_access_id_file: str,
    blackpearl_map_entries: List[BlackPearlMapEntry],
    user_without_mapfile: str,
) -> None:
    map_parser = BlackPearlAccessIdFileParser(blackpearl_access_id_file)

    assert "nonexistant_user" not in blackpearl_map_entries
    with pytest.raises(KeyError):
        map_parser["nonexistant_user"]


def test_blackpearl_mapfile_parser_not_found(
    fs: FakeFilesystem,
) -> None:
    with pytest.raises(IOError):
        BlackPearlAccessIdFileParser("/not-existing-file")
